/*******************************************************************************
 * Copyright 2007 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace TestRunFailAssignment
{
    class Program
    {
        static void Main(string[] args)
        {
            //Let's define some variables we will use with SOAP 
            string currProject = System.Environment.GetEnvironmentVariable("PROJNAME");
            string soapUser = "Administrator";
            string soapUserPassword = "";
            string testRunFailed = System.Environment.GetEnvironmentVariable("TRNUM");
            

            long theCookie = 0; ;
            //define the ttsoapcgi object
            ttsoapcgi ttSoap;
            //creating an instance of this object
            ttSoap = new ttsoapcgi();
            //set the URL of the ttsoapcgi object
            ttSoap.Url = "http://localhost:80/scripts/ttsoapcgi.exe";

            try
            {
                //get the list of projects
                CProject[] projectList = ttSoap.getProjectList(soapUser, soapUserPassword);
                CProject pProject = null;

                for (int i = 0; i < projectList.Length; i++)
                {
                    //Let's see if any of the projects match the database name.
                    if (projectList[i].database.name == currProject)
                    {
                        pProject = projectList[i];
                    }
                }
                if (pProject != null)
                {
                    theCookie = ttSoap.ProjectLogon(pProject, soapUser, soapUserPassword);
                }
                else
                {
                    Logger.log("Project Name does not match database name!");
                }
            }
            catch (Exception err)
            {
                Logger.log(err.Message);
            }

            //Next, we need to get the TestRun object so we can take a look at its links
            try
            {
                long defNumber = 0;
                long trNumber = Convert.ToInt64(testRunFailed);
                //Define and instantiate Test Run object.
                CTestRun pTestRun = ttSoap.getTestRun(theCookie, trNumber, "", false);
                //Get TestCase Record ID that created Test Run
                long tcRecordID = pTestRun.testcaserecordid;
                //Now we get the links for the Test Case
                CLink[] tcLinks = ttSoap.getLinksForItem(theCookie, "Test Case", tcRecordID);
                if (tcLinks != null)
                {
                    //backward for loop to start checking the newer links
                    for (int k = (tcLinks.Length - 1); k >= 0; k--)
                    {
                        //Check each item in the link and identify the defect
                        for (int j = 0; j < tcLinks[k].childList.Length; j++)
                        {
                            if (tcLinks[k].childList[j].tablename == "Defect")
                            {
                                //if we find a defect, get the number and quit
                                defNumber = Convert.ToInt64(tcLinks[k].childList[j].number);
                                break;
                            }
                        }
                    }
                    if (defNumber > 0)
                    {
                        CDefect pDefect = ttSoap.editDefect(theCookie, defNumber, "", false);
                        //Define array to store the user that defect will be assigned to
                        string[] userList = new string[1];
                        //Go through event list and see if we find a Fix event - starting at the newest event
                        for (int l = (pDefect.eventlist.Length - 1); l >=0; l--)
                        {
                            string eventToTest = "Fix";

                            if (pDefect.eventlist[l].name == eventToTest)
                            {
                                userList.SetValue(pDefect.eventlist[l].user, 0);
                                CDefectEvent[] eventComment = new CDefectEvent[1];
                                eventComment[0] = new CDefectEvent();
                                eventComment[0].date = System.DateTime.Now;
                                eventComment[0].name = "Verify";
                                eventComment[0].resultingstate = "Open (Verify Failed)";
                                eventComment[0].notes = "Automated assignment: Test Run failed";
                                eventComment[0].user = "Administrator, System";
                                eventComment[0].assigntolist = userList;
                                pDefect.eventlist.SetValue(eventComment[0], pDefect.eventlist.Length - 1);
                                ttSoap.saveDefect(theCookie, pDefect);
                            }
                            
                        }

                        
                    }
                    else
                    {
                        Logger.log("Test Run not linked to defect.");
                    }
            
                }
                else
                {
                    Logger.log("Test Run not linked to defect.");
                }
            }
            catch (Exception fcc)
            {
                Logger.log(fcc.Message);
            }
            try
            {
                ttSoap.DatabaseLogoff(theCookie);


            }
            catch (Exception err)
            {
                Logger.log(err.Message);
            }
        }
    }
}
